@@ -5,11 +5,10 @@
 	 * tables, reports, arbitrary queries, etc. to avoid code duplication.
 	 * @param $query The SQL SELECT string to execute
 	 * @param $count The same SQL query, but only retrieves the count of the rows (AS total)
-	 * @param $return_url The return URL
-	 * @param $return_desc The return link name
+	 * @param $return The return section
 	 * @param $page The current page
 	 *
-	 * $Id: display.php,v 1.67 2008/02/11 21:28:36 ioguix Exp $
+	 * $Id: display.php,v 1.68 2008/04/14 12:44:27 ioguix Exp $
 	 */
 
 	// Prevent timeouts on large exports (non-safe mode only)
@@ -32,35 +31,23 @@
 		if (is_array($_REQUEST['key']))
            $key = $_REQUEST['key'];
         else
-           $key = unserialize($_REQUEST['key']);
+           $key = unserialize(urldecode($_REQUEST['key']));
 
 		if ($confirm) {
 			$misc->printTrail($_REQUEST['subject']);
 			$misc->printTitle($lang['streditrow']);
 			$misc->printMsg($msg);
 
-			$bAllowAC = ($conf['autocomplete'] != 'disable') ? TRUE : FALSE;
-			if($bAllowAC) {
-				$constraints = $data->getConstraints($_REQUEST['table']);
-				$arrayLocals = array();
-				$arrayRefs = array();
-				$nC = 0;
-				while(!$constraints->EOF) {
-					// The following RE will match a FK constrain with a single (quoted or not) referencing column. At the moment we don't support multicolumn FKs
-					preg_match('/^FOREIGN KEY \(("[^"]*"|[^\s",]*)\) REFERENCES (.*)\((.*)\)/i', $constraints->fields['consrc'], $matches);
-					if(!empty($matches)) {
-						// Strip possible quotes and save
-						$arrayLocals[$nC] = preg_replace('/"(.*)"/', '$1', $matches[1]);
-						$arrayRefs[$nC] = array(preg_replace('/"(.*)"/', '$1', $matches[2]), preg_replace('/"(.*)"/', '$1', $matches[3]));
-						$nC++;
-					}
-					$constraints->moveNext();
-				}
-			}
-
 			$attrs = $data->getTableAttributes($_REQUEST['table']);
 			$rs = $data->browseRow($_REQUEST['table'], $key);
-			
+
+			if (($conf['autocomplete'] != 'disable')) {
+				$fksprops = $misc->getAutocompleteFKProperties($_REQUEST['table']);
+				if ($fksprops !== false)
+					echo $fksprops['code'];
+			}
+			else $fksprops = false;
+
 			echo "<form action=\"display.php\" method=\"post\" id=\"ac_form\">\n";
 			$elements = 0;
 			$error = true;			
@@ -74,18 +61,7 @@
 
 				$i = 0;
 				while (!$attrs->EOF) {
-					$szValueName = "values[{$attrs->fields['attname']}]";
-					$szEvents = '';
-					$szDivPH = '';
-					if($bAllowAC) {
-						$idxFound = array_search($attrs->fields['attname'], $arrayLocals);
-						// In PHP < 4.2.0 array_search returns NULL on failure
-						if ($idxFound !== NULL && $idxFound !== FALSE) {
-							$szEvent = "makeAC('{$szValueName}',{$i},'{$arrayRefs[$idxFound][0]}','{$arrayRefs[$idxFound][1]}','{$_REQUEST['server']}','{$_REQUEST['database']}');";
-							$szEvents = "onfocus=\"{$szEvent}\" onblur=\"hideAC();document.getElementById('ac_form').onsubmit=function(){return true;};\" onchange=\"{$szEvent}\" id=\"{$szValueName}\" onkeyup=\"{$szEvent}\" autocomplete=\"off\" class='ac_field'";
-							$szDivPH = "<div id=\"fac{$i}_ph\"></div>";
-						}
-					}
+
 					$attrs->fields['attnotnull'] = $data->phpBool($attrs->fields['attnotnull']);
 					$id = (($i % 2) == 0 ? '1' : '2');
 					
@@ -93,20 +69,20 @@
 					if (!isset($_REQUEST['format'][$attrs->fields['attname']]))
 						$_REQUEST['format'][$attrs->fields['attname']] = 'VALUE';
 					
-					echo "<tr>\n";
-					echo "<td class=\"data{$id}\" style=\"white-space:nowrap;\">", $misc->printVal($attrs->fields['attname']), "</td>";
-					echo "<td class=\"data{$id}\" style=\"white-space:nowrap;\">\n";
+					echo "<tr class=\"data{$id}\">\n";
+					echo "<td style=\"white-space:nowrap;\">", $misc->printVal($attrs->fields['attname']), "</td>";
+					echo "<td style=\"white-space:nowrap;\">\n";
 					echo $misc->printVal($data->formatType($attrs->fields['type'], $attrs->fields['atttypmod']));
 					echo "<input type=\"hidden\" name=\"types[", htmlspecialchars($attrs->fields['attname']), "]\" value=\"", 
 						htmlspecialchars($attrs->fields['type']), "\" /></td>";
 					$elements++;
-					echo "<td class=\"data{$id}\" style=\"white-space:nowrap;\">\n";
+					echo "<td style=\"white-space:nowrap;\">\n";
 					echo "<select name=\"format[", htmlspecialchars($attrs->fields['attname']), "]\">\n";
 					echo "<option value=\"VALUE\"", ($_REQUEST['format'][$attrs->fields['attname']] == 'VALUE') ? ' selected="selected"' : '', ">{$lang['strvalue']}</option>\n";
 					echo "<option value=\"EXPRESSION\"", ($_REQUEST['format'][$attrs->fields['attname']] == 'EXPRESSION') ? ' selected="selected"' : '', ">{$lang['strexpression']}</option>\n";
 					echo "</select>\n</td>\n";
 					$elements++;
-					echo "<td class=\"data{$id}\" style=\"white-space:nowrap;\">";
+					echo "<td style=\"white-space:nowrap;\">";
 					// Output null box if the column allows nulls (doesn't look at CHECKs or ASSERTIONS)
 					if (!$attrs->fields['attnotnull']) {
 						// Set initial null values
@@ -120,18 +96,25 @@
 					else
 						echo "&nbsp;</td>";
 
-					echo "<td class=\"data{$id}\" id=\"aciwp{$i}\" style=\"white-space:nowrap;\">";
+					echo "<td id=\"row_att_{$attrs->fields['attnum']}\" style=\"white-space:nowrap;\">";
+
+					$extras = array();
+
 					// If the column allows nulls, then we put a JavaScript action on the data field to unset the
 					// NULL checkbox as soon as anything is entered in the field.  We use the $elements variable to 
 					// keep track of which element offset we're up to.  We can't refer to the null checkbox by name
 					// as it contains '[' and ']' characters.
 					if (!$attrs->fields['attnotnull']) {
-						echo $data->printField($szValueName, $rs->fields[$attrs->fields['attname']], $attrs->fields['type'], 
-													array('onChange' => 'elements[' . ($elements - 1) . '].checked = false;'),$szEvents) . $szDivPH;
+						$extras['onChange'] = 'elements[' . ($elements - 1) . '].checked = false;';
 					}
-					else {
-						echo $data->printField($szValueName, $rs->fields[$attrs->fields['attname']], $attrs->fields['type'],array(),$szEvents) . $szDivPH;
+
+					if (($fksprops !== false) && isset($fksprops['byfield'][$attrs->fields['attnum']])) {
+						$extras['id'] = "attr_{$attrs->fields['attnum']}";
+						$extras['autocomplete'] = 'off';
 					}
+
+					echo $data->printField("values[{$attrs->fields['attname']}]", $rs->fields[$attrs->fields['attname']], $attrs->fields['type'], $extras);
+
 					echo "</td>";
 					$elements++;
 					echo "</tr>\n";
@@ -139,10 +122,7 @@
 					$attrs->moveNext();
 				}
 				echo "</table>\n";
-				if($bAllowAC) {
-					echo '<script src="aciur.js" type="text/javascript"></script>';
-					echo "<div id=\"ac\"></div>";
-				}
+
 				$error = false;
 			}
 			elseif ($rs->recordCount() != 1) {
@@ -162,32 +142,33 @@
 				echo "<input type=\"hidden\" name=\"query\" value=\"", htmlspecialchars($_REQUEST['query']), "\" />\n";
 			if (isset($_REQUEST['count']))
 				echo "<input type=\"hidden\" name=\"count\" value=\"", htmlspecialchars($_REQUEST['count']), "\" />\n";
-			if (isset($_REQUEST['return_url']))
-				echo "<input type=\"hidden\" name=\"return_url\" value=\"", htmlspecialchars($_REQUEST['return_url']), "\" />\n";
-			if (isset($_REQUEST['return_desc']))
-				echo "<input type=\"hidden\" name=\"return_desc\" value=\"", htmlspecialchars($_REQUEST['return_desc']), "\" />\n";
+			if (isset($_REQUEST['return']))
+				echo "<input type=\"hidden\" name=\"return\" value=\"", htmlspecialchars($_REQUEST['return']), "\" />\n";
 			echo "<input type=\"hidden\" name=\"page\" value=\"", htmlspecialchars($_REQUEST['page']), "\" />\n";
 			echo "<input type=\"hidden\" name=\"sortkey\" value=\"", htmlspecialchars($_REQUEST['sortkey']), "\" />\n";
 			echo "<input type=\"hidden\" name=\"sortdir\" value=\"", htmlspecialchars($_REQUEST['sortdir']), "\" />\n";
 			echo "<input type=\"hidden\" name=\"strings\" value=\"", htmlspecialchars($_REQUEST['strings']), "\" />\n";
-			echo "<input type=\"hidden\" name=\"key\" value=\"", htmlspecialchars(serialize($key)), "\" />\n";
+			echo "<input type=\"hidden\" name=\"key\" value=\"", htmlspecialchars(urlencode(serialize($key))), "\" />\n";
 			echo "<p>";
 			if (!$error) echo "<input type=\"submit\" name=\"save\" value=\"{$lang['strsave']}\" />\n";
 			echo "<input type=\"submit\" name=\"cancel\" value=\"{$lang['strcancel']}\" />\n";
-			if($bAllowAC) {
-				$szChecked = $conf['autocomplete'] != 'default off' ? 'checked="checked"' : '';
-				echo "<input type=\"checkbox\" name=\"no_ac\" id=\"no_ac\" onclick=\"rEB(this.checked);\" value=\"1\" {$szChecked} /><label for='no_ac' onmouseover='this.style.cursor=\"pointer\";'>{$lang['strac']}</label>\n";
+
+			if($fksprops !== false) {
+				if ($conf['autocomplete'] != 'default off')
+					echo "<input type=\"checkbox\" id=\"no_ac\" value=\"1\" checked=\"checked\" /><label for=\"no_ac\">{$lang['strac']}</label>\n";
+				else
+					echo "<input type=\"checkbox\" id=\"no_ac\" value=\"0\" /><label for=\"no_ac\">{$lang['strac']}</label>\n";
 			}
+
 			echo "</p>\n";
 			echo "</form>\n";
-			echo "<script>rEB(document.getElementById('no_ac').checked);</script>";
 		}
 		else {
 			if (!isset($_POST['values'])) $_POST['values'] = array();
 			if (!isset($_POST['nulls'])) $_POST['nulls'] = array();
 			
 			$status = $data->editRow($_POST['table'], $_POST['values'], $_POST['nulls'], 
-												$_POST['format'], $_POST['types'], unserialize($_POST['key']));
+				$_POST['format'], $_POST['types'], $key);
 			if ($status == 0)
 				doBrowse($lang['strrowupdated']);
 			elseif ($status == -2)
@@ -222,21 +203,19 @@
 				echo "<input type=\"hidden\" name=\"query\" value=\"", htmlspecialchars($_REQUEST['query']), "\" />\n";
 			if (isset($_REQUEST['count']))
 				echo "<input type=\"hidden\" name=\"count\" value=\"", htmlspecialchars($_REQUEST['count']), "\" />\n";
-			if (isset($_REQUEST['return_url']))
-				echo "<input type=\"hidden\" name=\"return_url\" value=\"", htmlspecialchars($_REQUEST['return_url']), "\" />\n";
-			if (isset($_REQUEST['return_desc']))
-				echo "<input type=\"hidden\" name=\"return_desc\" value=\"", htmlspecialchars($_REQUEST['return_desc']), "\" />\n";
+			if (isset($_REQUEST['return']))
+				echo "<input type=\"hidden\" name=\"return\" value=\"", htmlspecialchars($_REQUEST['return']), "\" />\n";
 			echo "<input type=\"hidden\" name=\"page\" value=\"", htmlspecialchars($_REQUEST['page']), "\" />\n";
 			echo "<input type=\"hidden\" name=\"sortkey\" value=\"", htmlspecialchars($_REQUEST['sortkey']), "\" />\n";
 			echo "<input type=\"hidden\" name=\"sortdir\" value=\"", htmlspecialchars($_REQUEST['sortdir']), "\" />\n";
 			echo "<input type=\"hidden\" name=\"strings\" value=\"", htmlspecialchars($_REQUEST['strings']), "\" />\n";
-			echo "<input type=\"hidden\" name=\"key\" value=\"", htmlspecialchars(serialize($_REQUEST['key'])), "\" />\n";
+			echo "<input type=\"hidden\" name=\"key\" value=\"", htmlspecialchars(urlencode(serialize($_REQUEST['key']))), "\" />\n";
 			echo "<input type=\"submit\" name=\"yes\" value=\"{$lang['stryes']}\" />\n";
 			echo "<input type=\"submit\" name=\"no\" value=\"{$lang['strno']}\" />\n";
 			echo "</form>\n";
 		}
 		else {
-			$status = $data->deleteRow($_POST['table'], unserialize($_POST['key']));
+			$status = $data->deleteRow($_POST['table'], unserialize(urldecode($_POST['key'])));
 			if ($status == 0)
 				doBrowse($lang['strrowdeleted']);
 			elseif ($status == -2)
@@ -246,13 +225,173 @@
 		}
 		
 	}
+	
+	/* build & return the FK information data structure 
+	 * used when deciding if a field should have a FK link or not*/
+	function &getFKInfo() {
+		global $data, $misc, $lang;
+		 
+		// Get the foreign key(s) information from the current table
+		$fkey_information = array('byconstr' => array(), 'byfield' => array());
+
+		if (isset($_REQUEST['table'])) {
+			$constraints = $data->getConstraintsWithFields($_REQUEST['table']);
+			if ($constraints->recordCount() > 0) {
+
+				$fkey_information['common_url'] = $misc->getHREF('schema') .'&amp;subject=table';
+
+				/* build the FK constraints data structure */
+				while (!$constraints->EOF) {
+					$constr =& $constraints->fields;
+					if ($constr['contype'] == 'f') {
+
+						if (!isset($fkey_information['byconstr'][$constr['conid']])) {
+							$fkey_information['byconstr'][$constr['conid']] = array (
+								'url_data' => 'table='. urlencode($constr['f_table']) .'&amp;schema='. urlencode($constr['f_schema']),
+								'fkeys' => array(),
+								'consrc' => $constr['consrc']
+							);
+						}
+
+						$fkey_information['byconstr'][$constr['conid']]['fkeys'][$constr['p_field']] = $constr['f_field'];
+
+						if (!isset($fkey_information['byfield'][$constr['p_field']]))
+							$fkey_information['byfield'][$constr['p_field']] = array();
+
+						$fkey_information['byfield'][$constr['p_field']][] = $constr['conid'];
+					}
+					$constraints->moveNext();
+				}
+			}
+		}
+
+		return $fkey_information;
+	}
+
+	/* Print table header cells 
+	 * @param $sortLink must be urlencoded already
+	 * */
+	function printTableHeaderCells(&$rs, $sortLink, $withOid) {
+		global $misc, $data, $conf;
+		$j = 0;
+
+		foreach ($rs->fields as $k => $v) {
+
+			if (($k === $data->id) && ( !($withOid && $conf['show_oids']) )) {
+				$j++;
+				continue;
+			}
+			$finfo = $rs->fetchField($j);
+
+			if ($sortLink === false) {
+				echo "<th class=\"data\">", $misc->printVal($finfo->name), "</th>\n";
+			}
+			else {
+				echo "<th class=\"data\"><a href=\"display.php?{$sortLink}&amp;sortkey=", ($j + 1), "&amp;sortdir=";
+				// Sort direction opposite to current direction, unless it's currently ''
+				echo ($_REQUEST['sortdir'] == 'asc' && $_REQUEST['sortkey'] == ($j + 1)) ? 'desc' : 'asc';
+				echo "&amp;strings=", urlencode($_REQUEST['strings']), 
+					"&amp;page=" . urlencode($_REQUEST['page']), "\">", 
+					$misc->printVal($finfo->name), "</a></th>\n";
+			}
+			$j++;
+		}
+
+		reset($rs->fields);
+	}
+
+	/* Print data-row cells */
+	function printTableRowCells(&$rs, &$fkey_information, $withOid) {
+		global $data, $misc, $conf;
+		$j = 0;
+		
+		if (!isset($_REQUEST['strings'])) $_REQUEST['strings'] = 'collapsed';
+
+		foreach ($rs->fields as $k => $v) {
+			$finfo = $rs->fetchField($j++);
+
+			if (($k === $data->id) && ( !($withOid && $conf['show_oids']) )) continue;
+			elseif ($v !== null && $v == '') echo "<td>&nbsp;</td>";
+			else {
+				echo "<td style=\"white-space:nowrap;\">";
+
+				if (($v !== null) && isset($fkey_information['byfield'][$k])) {
+					foreach ($fkey_information['byfield'][$k] as $conid) {
+
+						$query_params = $fkey_information['byconstr'][$conid]['url_data'];
+
+						foreach ($fkey_information['byconstr'][$conid]['fkeys'] as $p_field => $f_field) {
+							$query_params .= '&amp;'. urlencode("fkey[{$f_field}]") .'='. urlencode($rs->fields[$p_field]);
+						}
+
+						/* $fkey_information['common_url'] is already urlencoded */
+						$query_params .= '&amp;'. $fkey_information['common_url'];
+						echo "<div style=\"display:inline-block;\">";
+						echo "<a class=\"fk fk_". htmlentities($conid) ."\" href=\"display.php?{$query_params}\">";
+						echo "<img src=\"".$misc->icon('ForeignKey')."\" style=\"vertical-align:middle;\" alt=\"[fk]\" title=\""
+							. htmlentities($fkey_information['byconstr'][$conid]['consrc'])
+							."\" />";
+						echo "</a>";
+						echo "</div>";
+					}
+					echo $misc->printVal($v, $finfo->type, array('null' => true, 'clip' => ($_REQUEST['strings']=='collapsed'), 'class' => 'fk_value'));
+				} else {
+					echo $misc->printVal($v, $finfo->type, array('null' => true, 'clip' => ($_REQUEST['strings']=='collapsed')));
+				}
+				echo "</td>";
+			}
+		}
+	}
+
+	/* Print the FK row, used in ajax requests */
+	function doBrowseFK() {
+		global $data, $misc, $lang;
+
+		$ops = array();
+		foreach($_REQUEST['fkey'] as $x => $y) {
+			$ops[$x] = '=';
+		}
+		$query = $data->getSelectSQL($_REQUEST['table'], array(), $_REQUEST['fkey'], $ops);
+		$_REQUEST['query'] = $query;
+
+		$fkinfo =& getFKInfo();
+
+		$max_pages = 1;
+		// Retrieve page from query.  $max_pages is returned by reference.
+		$rs = $data->browseQuery('SELECT', $_REQUEST['table'], $_REQUEST['query'],  
+			null, null, 1, 1, $max_pages);
+
+		echo "<a href=\"\" style=\"display:table-cell;\" class=\"fk_delete\"><img alt=\"[delete]\" src=\"". $misc->icon('Delete') ."\" /></a>\n";
+		echo "<div style=\"display:table-cell;\">";
+
+		if (is_object($rs) && $rs->recordCount() > 0) {
+			/* we are browsing a referenced table here
+			 * we should show OID if show_oids is true
+			 * so we give true to withOid in functions bellow
+			 * as 3rd paramter */
+		
+			echo "<table><tr>";
+				printTableHeaderCells($rs, false, true);
+			echo "</tr>";
+			echo "<tr class=\"data1\">\n";
+				printTableRowCells($rs, $fkinfo, true);
+			echo "</tr>\n";
+			echo "</table>\n";
+		}
+		else
+			echo $lang['strnodata'];
+
+		echo "</div>";
+
+		exit;
+	}
 
 	/** 
 	 * Displays requested data
 	 */
 	function doBrowse($msg = '') {
 		global $data, $conf, $misc, $lang;
-		
+
 		$save_history = false;
 		// If current page is not set, default to first page
 		if (!isset($_REQUEST['page']))
@@ -267,14 +406,30 @@
 		else {
 			$subject = '';
 		}
-	
+
 		$misc->printTrail(isset($subject) ? $subject : 'database');
+
+		/* This code is used when browsing FK in pure-xHTML (without js) */
+		if (isset($_REQUEST['fkey'])) {
+			$ops = array();
+			foreach($_REQUEST['fkey'] as $x => $y) {
+				$ops[$x] = '=';
+			}
+			$query = $data->getSelectSQL($_REQUEST['table'], array(), $_REQUEST['fkey'], $ops);
+			$_REQUEST['query'] = $query;
+		}
 		
 		if (isset($object)) {
 			if (isset($_REQUEST['query'])) {
+				$_SESSION['sqlquery'] = $_REQUEST['query'];
 				$misc->printTitle($lang['strselect']);
 				$type = 'SELECT';
-			} else {
+			}
+			else if (isset($_REQUEST['report'])) {
+				$misc->printTitle($lang['strselect']);
+				$type = 'SELECT';
+			}
+			else {
 				$misc->printTitle($lang['strbrowse']);
 				$type = 'TABLE';
 			}
@@ -287,7 +442,7 @@
 
 		// If 'sortkey' is not set, default to ''
 		if (!isset($_REQUEST['sortkey'])) $_REQUEST['sortkey'] = '';
-	
+
 		// If 'sortdir' is not set, default to ''
 		if (!isset($_REQUEST['sortdir'])) $_REQUEST['sortdir'] = '';
 	
@@ -301,7 +456,7 @@
 			$key = array();
 		
 		// Set the schema search path
-		if ($data->hasSchemas() && isset($_REQUEST['search_path'])) {
+		if (isset($_REQUEST['search_path'])) {
 			if ($data->setSearchPath(array_map('trim',explode(',',$_REQUEST['search_path']))) != 0) {
 				return;
 			}
@@ -310,32 +465,36 @@
 		// Retrieve page from query.  $max_pages is returned by reference.
 		$rs = $data->browseQuery($type, 
 			isset($object) ? $object : null, 
-			isset($_REQUEST['query']) ? $_REQUEST['query'] : null, 
+			isset($_SESSION['sqlquery']) ? $_SESSION['sqlquery'] : null,
 			$_REQUEST['sortkey'], $_REQUEST['sortdir'], $_REQUEST['page'],
 			$conf['max_rows'], $max_pages);
-	
+
+		$fkey_information =& getFKInfo();
+
 		// Build strings for GETs
-		$str = 	$misc->href; // . "&amp;page=" . urlencode($_REQUEST['page']);
-		if (isset($object)) $str .= "&amp;" . urlencode($subject) . '=' . urlencode($object);
-		if (isset($subject)) $str .= "&amp;subject=" . urlencode($subject);
-		if (isset($_REQUEST['query'])) $str .= "&amp;query=" . urlencode($_REQUEST['query']);
-		if (isset($_REQUEST['count'])) $str .= "&amp;count=" . urlencode($_REQUEST['count']);
-		if (isset($_REQUEST['return_url'])) $str .= "&amp;return_url=" . urlencode($_REQUEST['return_url']);
-		if (isset($_REQUEST['return_desc'])) $str .= "&amp;return_desc=" . urlencode($_REQUEST['return_desc']);
-		if (isset($_REQUEST['search_path'])) $str .= "&amp;search_path=" . urlencode($_REQUEST['search_path']);
+		$gets = $misc->href;
+		if (isset($object)) $gets .= "&amp;" . urlencode($subject) . '=' . urlencode($object);
+		if (isset($subject)) $gets .= "&amp;subject=" . urlencode($subject);
+		if (isset($_REQUEST['query'])) $gets .= "&amp;query=" . urlencode($_REQUEST['query']);
+		if (isset($_REQUEST['report'])) $gets .= "&amp;report=" . urlencode($_REQUEST['report']);
+		if (isset($_REQUEST['count'])) $gets .= "&amp;count=" . urlencode($_REQUEST['count']);
+		if (isset($_REQUEST['return'])) $gets .= "&amp;return=" . urlencode($_REQUEST['return']);
+		if (isset($_REQUEST['search_path'])) $gets .= "&amp;search_path=" . urlencode($_REQUEST['search_path']);
+		if (isset($_REQUEST['table'])) $gets .= "&amp;table=" . urlencode($_REQUEST['table']);
 		
 		// This string just contains sort info
-		$str2 = "sortkey=" . urlencode($_REQUEST['sortkey']) . 
+		$getsort = "sortkey=" . urlencode($_REQUEST['sortkey']) .
 			"&amp;sortdir=" . urlencode($_REQUEST['sortdir']);
-			
+
 		if ($save_history && is_object($rs) && ($type == 'QUERY')) //{
 			$misc->saveScriptHistory($_REQUEST['query']);
 
 		if (is_object($rs) && $rs->recordCount() > 0) {
 			// Show page navigation
-			$misc->printPages($_REQUEST['page'], $max_pages, "display.php?page=%s&amp;{$str}&amp;{$str2}&amp;nohistory=t&amp;strings=" . urlencode($_REQUEST['strings']));
-			echo "<table>\n<tr>";
-	
+			$misc->printPages($_REQUEST['page'], $max_pages, "display.php?page=%s&amp;{$gets}&amp;{$getsort}&amp;nohistory=t&amp;strings=" . urlencode($_REQUEST['strings']));
+
+			echo "<table id=\"data\">\n<tr>";
+
 			// Check that the key is actually in the result set.  This can occur for select
 			// operations where the key fields aren't part of the select.  XXX:  We should
 			// be able to support this, somehow.
@@ -351,36 +510,16 @@
 			if (sizeof($key) > 0)
 				echo "<th colspan=\"2\" class=\"data\">{$lang['stractions']}</th>\n";
 
-			$j = 0;		
-			foreach ($rs->fields as $k => $v) {
-				if (isset($object) && $k == $data->id && !$conf['show_oids']) {
-					$j++;
-					continue;
-				}
-				$finfo = $rs->fetchField($j);
-				// Display column headers with sorting options, unless we're PostgreSQL
-				// 7.0 and it's a non-TABLE mode
-				if (!$data->hasFullSubqueries() && $type != 'TABLE') {
-					echo "<th class=\"data\">", $misc->printVal($finfo->name), "</th>\n";
-				}
-				else {
-					echo "<th class=\"data\"><a href=\"display.php?{$str}&amp;sortkey=", ($j + 1), "&amp;sortdir=";
-					// Sort direction opposite to current direction, unless it's currently ''
-					echo ($_REQUEST['sortdir'] == 'asc' && $_REQUEST['sortkey'] == ($j + 1)) ? 'desc' : 'asc';
-					echo "&amp;strings=", urlencode($_REQUEST['strings']), 
-						"&amp;page=" . urlencode($_REQUEST['page']), "\">", 
-						$misc->printVal($finfo->name), "</a></th>\n";
-				}
-				$j++;
-			}
-	
+			/* we show OIDs only if we are in TABLE or SELECT type browsing */
+			printTableHeaderCells($rs, $gets, isset($object));
+
 			echo "</tr>\n";
-	
+
 			$i = 0;		
 			reset($rs->fields);
 			while (!$rs->EOF) {
 				$id = (($i % 2) == 0 ? '1' : '2');
-				echo "<tr>\n";
+				echo "<tr class=\"data{$id}\">\n";
 				// Display edit and delete links if we have a key
 				if (sizeof($key) > 0) {
 					$key_str = '';
@@ -394,42 +533,41 @@
 						$key_str .= urlencode("key[{$v}]") . '=' . urlencode($rs->fields[$v]);
 					}
 					if ($has_nulls) {
-						echo "<td class=\"data{$id}\" colspan=\"2\">&nbsp;</td>\n";
+						echo "<td colspan=\"2\">&nbsp;</td>\n";
 					} else {
 						echo "<td class=\"opbutton{$id}\"><a href=\"display.php?action=confeditrow&amp;strings=", 
 							urlencode($_REQUEST['strings']), "&amp;page=", 
-							urlencode($_REQUEST['page']), "&amp;{$key_str}&amp;{$str}&amp;{$str2}\">{$lang['stredit']}</a></td>\n";
+							urlencode($_REQUEST['page']), "&amp;{$key_str}&amp;{$gets}&amp;{$getsort}\">{$lang['stredit']}</a></td>\n";
 						echo "<td class=\"opbutton{$id}\"><a href=\"display.php?action=confdelrow&amp;strings=", 
 							urlencode($_REQUEST['strings']), "&amp;page=", 
-							urlencode($_REQUEST['page']), "&amp;{$key_str}&amp;{$str}&amp;{$str2}\">{$lang['strdelete']}</a></td>\n";
-					}
-				}
-				$j = 0;
-				foreach ($rs->fields as $k => $v) {
-					$finfo = $rs->fetchField($j++);
-					if (isset($_REQUEST['table']) && $k == $data->id && !$conf['show_oids']) continue;
-					elseif ($v !== null && $v == '') echo "<td class=\"data{$id}\">&nbsp;</td>";
-					else {
-						echo "<td class=\"data{$id}\" style=\"white-space:nowrap;\">",
-							$misc->printVal($v, $finfo->type, array('null' => true, 'clip' => ($_REQUEST['strings']=='collapsed'))), "</td>";
+							urlencode($_REQUEST['page']), "&amp;{$key_str}&amp;{$gets}&amp;{$getsort}\">{$lang['strdelete']}</a></td>\n";
 					}
 				}
+
+				print printTableRowCells($rs, $fkey_information, isset($object));
+
 				echo "</tr>\n";
 				$rs->moveNext();
 				$i++;
 			}
-			echo "</table>\n";			
+			echo "</table>\n";
+
 			echo "<p>", $rs->recordCount(), " {$lang['strrows']}</p>\n";
 			// Show page navigation
-			$misc->printPages($_REQUEST['page'], $max_pages, "display.php?page=%s&amp;{$str}&amp;{$str2}&amp;strings=" . urlencode($_REQUEST['strings']));
+			$misc->printPages($_REQUEST['page'], $max_pages, "display.php?page=%s&amp;{$gets}&amp;{$getsort}&amp;strings=" . urlencode($_REQUEST['strings']));
 		}
 		else echo "<p>{$lang['strnodata']}</p>\n";
 
 		// Navigation links	
 		echo "<ul class=\"navlink\">\n";
+
 		// Return
-		if (isset($_REQUEST['return_url']) && isset($_REQUEST['return_desc']))
-			echo "\t<li><a href=\"{$_REQUEST['return_url']}\">{$_REQUEST['return_desc']}</a></li>\n";
+		if (isset($_REQUEST['return'])) {
+			$return_url = $misc->getHREFSubject($_REQUEST['return']);
+
+			if ($return_url)
+				echo "\t<li><a href=\"{$return_url}\">{$lang['strback']}</a></li>\n";
+		}
 
 		// Edit SQL link
 		if (isset($_REQUEST['query']))
@@ -438,16 +576,16 @@
 
 		// Expand/Collapse
 		if ($_REQUEST['strings'] == 'expanded')
-			echo "\t<li><a href=\"display.php?{$str}&amp;{$str2}&amp;strings=collapsed&amp;page=", 
+			echo "\t<li><a href=\"display.php?{$gets}&amp;{$getsort}&amp;strings=collapsed&amp;page=", 
 				urlencode($_REQUEST['page']), "\">{$lang['strcollapse']}</a></li>\n";
 		else
-			echo "\t<li><a href=\"display.php?{$str}&amp;{$str2}&amp;strings=expanded&amp;page=", 
+			echo "\t<li><a href=\"display.php?{$gets}&amp;{$getsort}&amp;strings=expanded&amp;page=", 
 				urlencode($_REQUEST['page']), "\">{$lang['strexpand']}</a></li>\n";
 
 		// Create report
 		if (isset($_REQUEST['query']) && ($subject !== 'report') && $conf['show_reports'] && isset($rs) && is_object($rs) && $rs->recordCount() > 0)
 			echo "\t<li><a href=\"reports.php?{$misc->href}&amp;action=create&amp;report_sql=",
-				urlencode($_REQUEST['query']), "&amp;paginate=", urlencode($_REQUEST['paginate']), "\">{$lang['strcreatereport']}</a></li>\n";
+				urlencode($_REQUEST['query']), "&amp;paginate=", (isset($_REQUEST['paginate'])? urlencode($_REQUEST['paginate']):'f'), "\">{$lang['strcreatereport']}</a></li>\n";
 
 		// Create view and download
 		if (isset($_REQUEST['query']) && isset($rs) && is_object($rs) && $rs->recordCount() > 0) {
@@ -467,15 +605,30 @@
 				urlencode($object), "&amp;{$misc->href}\">{$lang['strinsert']}</a></li>\n";
 
 		// Refresh
-		echo "\t<li><a href=\"display.php?{$str}&amp;{$str2}&amp;strings=", urlencode($_REQUEST['strings']), 
+		echo "\t<li><a href=\"display.php?{$gets}&amp;{$getsort}&amp;strings=", urlencode($_REQUEST['strings']), 
 			"&amp;page=" . urlencode($_REQUEST['page']),
 			"\">{$lang['strrefresh']}</a></li>\n";
 		echo "</ul>\n";
 	}
-	
+
+
+	/* shortcuts: this function exit the script for ajax purpose */
+	if ($action == 'dobrowsefk') {
+		doBrowseFK();
+	}
+
+	$scripts  = "<script src=\"libraries/js/jquery.js\" type=\"text/javascript\"></script>\n";
+	$scripts .= "<script src=\"js/display.js\" type=\"text/javascript\"></script>";
+
+	$scripts .= "<script type=\"text/javascript\">\n";
+	$scripts .= "var Display = {\n";
+	$scripts .= "errmsg: '". str_replace("'", "\'", $lang['strconnectionfail']) ."'\n";
+	$scripts .= "};\n";
+	$scripts .= "</script>\n";
+
 	// If a table is specified, then set the title differently
 	if (isset($_REQUEST['subject']) && isset($_REQUEST[$_REQUEST['subject']]))
-		$misc->printHeader($lang['strtables']);
+		$misc->printHeader($lang['strtables'], $scripts);
 	else	
 		$misc->printHeader($lang['strqueryresults']);
 
@@ -495,7 +648,7 @@
 			break;
 		case 'confdelrow':
 			doDelRow(true);
-			break;			
+			break;
 		default:
 			doBrowse();
 			break;
